#import <Foundation/Foundation.h>

@interface RPMHTTPResponse : NSObject
{
    NSHTTPURLResponse* _response;
    NSData*            _data;
    NSError*           _error;
}

@property (readonly) NSHTTPURLResponse* response;
@property (readonly) NSData*            data;
@property (readonly) NSError*           error;

- (id)initWithResponse:(NSHTTPURLResponse *)response data:(NSData *)data error:(NSError *)error;
+ (id)responseWithResponse:(NSHTTPURLResponse *)response data:(NSData *)data error:(NSError *)error;

- (NSInteger)statusCode;
- (NSString *)localizedStatusCode;
- (NSInteger)errorCode;
- (NSString *)MIMEType;

@end

@class RPMHTTPConnection;

@protocol RPMHTTPConnectionDelegate <NSObject>
@optional
- (void)connection:(RPMHTTPConnection *)connection finishedWithResponse:(RPMHTTPResponse *)response;
@end

@interface RPMHTTPConnection : NSObject
{
    NSURLConnection* _connection;
    NSThread*        _loadThread;
    BOOL             _wasCancelled;
    BOOL             _isLoading;
    
    NSHTTPURLResponse* _response;
    NSMutableData*     _data;
    NSError*           _error;
    
    id <RPMHTTPConnectionDelegate> _delegate;
}

@property (readonly) NSHTTPURLResponse *response;
@property (readonly) NSMutableData     *data;
@property (readonly) NSError           *error;

+ (RPMHTTPResponse *)sendSynchronousRequest:(NSURLRequest *)request;

- (id)initWithRequest:(NSURLRequest *)request delegate:(id <RPMHTTPConnectionDelegate>)delegate;
- (void)waitUntilCompletion;
- (void)cancel;

@end
